// async_reader - A simple buffered input source.

#include "async_reader.h"

void AsyncReader::getInt(std::function<void(int)> cb)
{
	boost::asio::dispatch(m_strand, [cb = std::move(cb), this]() mutable {
		// need to get more data from readSomeInts
		if (m_buffer.empty())
		{
			doReadSome([this, cb = std::move(cb)]() mutable {
				if (m_buffer.empty())
					throw std::runtime_error{"getInt requested but nothing was available"};

				cb(m_buffer.front());
				m_buffer.pop();
			});
		}
		// else can pull an int from the queue before reading more in
		else
		{
			cb(m_buffer.front());
			m_buffer.pop();
		}
	});
}

// Automatically keep reading into the buffer until it's full
void AsyncReader::doReadSome(std::function<void()> cb)
{
    boost::asio::defer(m_strand, [cb = std::move(cb), this]() mutable {
		// do nothing if the buffer is full already
		if (m_buffer.size() >= c_max_buffer_size)
			return;

		auto to_read = c_max_buffer_size - m_buffer.size();
		int more_ints[to_read];
		auto num_read = readSomeInts(more_ints, to_read);

		for (auto i = 0; i < num_read; i++)
		{
			m_buffer.push(more_ints[i]);
		}

		cb();
		
		boost::asio::defer(m_strand, [this](){
			doReadSome([](){});
		});
    });
}
